import 'dart:convert';

import 'package:dio/dio.dart';
import 'package:flutter/material.dart';
import 'package:onyva_mobile/core/exceptions/remote_repository_exception.dart';
import 'package:onyva_mobile/core/models/repositories/remote/remote_repository_exception_type.dart';
import 'package:onyva_mobile/core/models/repositories/remote/remote_repository_response.dart';
import 'package:onyva_mobile/core/repositories/remote/remote_repository_adapter.dart';

class RemoteRepositoryAdapterImpl implements RemoteRepositoryAdapter {
  static const String NO_INTERNET_LIB_MESSAGE =
      'SocketException: Failed host lookup';

  final Map<String, String> _headers = {
    'Content-type': 'application/json',
  };

  final _remoteRepositoryLib = Dio();

  @protected
  @override
  Future<RemoteRepositoryResponse<R>> delete<R>(
    String url,
  ) {
    // TODO: implement delete
    throw UnimplementedError();
  }

  @override
  Future<RemoteRepositoryResponse<R>> get<R>(
    String url, {
    Map<String, dynamic> queryParameters,
    void Function(int current, int total) onReceiveProgress,
  }) async {
    try {
      Response<R> result = await this._remoteRepositoryLib.get(
            '$url',
            queryParameters: queryParameters,
            onReceiveProgress: onReceiveProgress,
            options: Options(
              followRedirects: false,
              validateStatus: (status) {
                return status < 500;
              },
              headers: _headers,
            ),
          );

      return RemoteRepositoryResponse(
        statusCode: result.statusCode,
        responseBody: result.data,
      );
    } on DioError catch (e) {
      throw _decodeFromLibException(e);
    }
  }

  @override
  Future<RemoteRepositoryResponse<R>> patch<R>(
    String url, [
    dynamic data,
  ]) async {
    try {
      Response<R> result = await this._remoteRepositoryLib.patch(
            '$url',
            data: json.encode(data),
            options: Options(
              followRedirects: false,
              validateStatus: (status) {
                return status < 500;
              },
              headers: _headers,
            ),
          );

      return RemoteRepositoryResponse(
        statusCode: result.statusCode,
        responseBody: result.data,
      );
    } on DioError catch (e) {
      throw _decodeFromLibException(e);
    }
  }

  @override
  Future<RemoteRepositoryResponse<R>> post<R>(
    String url,
    dynamic data, {
    void Function(int current, int total) onSendProgress,
    Map<String, dynamic> queryParameters,
  }) async {
    try {
      Response<R> result = await this._remoteRepositoryLib.post(
            '$url',
            data: json.encode(data),
            options: Options(
              followRedirects: false,
              validateStatus: (status) {
                return status < 500;
              },
              headers: _headers,
            ),
            onSendProgress: onSendProgress,
            queryParameters: queryParameters,
          );

      return RemoteRepositoryResponse(
        statusCode: result.statusCode,
        responseBody: result.data,
      );
    } on DioError catch (e) {
      throw _decodeFromLibException(e);
    }
  }

  RemoteRepositoryException _decodeFromLibException(DioError e) {
    String message;
    RemoteRepositoryExceptionType exceptionType;

    if (e.type == DioErrorType.RESPONSE) {
      switch (e.response.statusCode) {
        case 401:
          message = e.message;
          exceptionType = RemoteRepositoryExceptionType.UNAUTHORIZED;
          break;

        case 400:
          print(e.response.data);
          message = (json.decode(e.response.data)['message']['details'] as List)
                  .first['message'] ??
              json.decode(e.response.data)['error'];
          exceptionType = RemoteRepositoryExceptionType.BAD_REQUEST;
          break;

        case 500:
          message = e.message;
          exceptionType = RemoteRepositoryExceptionType.INTERNAL_SERVER_ERROR;
          break;
      }
    } else if (e.type == DioErrorType.DEFAULT) {
      if (e.message.contains(NO_INTERNET_LIB_MESSAGE)) {
        message = e.message;
        exceptionType = RemoteRepositoryExceptionType.NO_INTERNET_CONNECTION;
      }
    }

    return RemoteRepositoryException(
      remoteRepositoryExceptionType: exceptionType,
      originalMessage: message,
    );
  }
}
