import 'dart:async';

import 'package:flutter/material.dart';
import 'package:onyva_mobile/core/widgets/block_ui.dart';
import 'package:onyva_mobile/business/authentication_business.dart';
import 'package:onyva_mobile/business/implementation/authentication_business_impl.dart';
import 'package:onyva_mobile/core/functions/generic_error_alert.dart';
import 'package:onyva_mobile/app_routes.dart';
import 'package:onyva_mobile/core/utils/functions.dart';
import 'package:onyva_mobile/models/enums/login_type.dart';
import 'package:onyva_mobile/state_managers/app_store.dart';
import 'package:onyva_mobile/state_managers/login/login_store.dart';
import 'package:onyva_mobile/viewmodel/login_viewmodel.dart';
import 'package:provider/provider.dart';

class LoginViewModelImpl implements LoginViewModel {
  final _emailCtrl = TextEditingController();
  final _passwordCtrl = TextEditingController();
  final _loginStore = LoginStore();
  final _blockUiStreamController = StreamController<bool>.broadcast();
  final AuthenticationBusiness _business = AuthenticationBusinessImpl();

  @override
  void toggleObscureText() {
    this._loginStore.toggleShowPassword();
  }

  @override
  bool get showPassword => this._loginStore.showPassword;

  @override
  bool get isLoading => this._loginStore.isLoading;

  @override
  TextEditingController get emailCtrl => _emailCtrl;

  @override
  TextEditingController get passwordCtrl => _passwordCtrl;

  @override
  void login(
    BuildContext context,
    LoginType loginType,
  ) async {
    blockUi(
      context: context,
      stream: _blockUiStreamController.stream,
    );

    _business
        .authenticateUser(
      loginType,
      email: _emailCtrl.text?.trim(),
      password: _passwordCtrl.text?.trim(),
    )
        .then((owner) {
      _blockUiStreamController.add(false);

      if (owner == null) return;

      AppStore appStore = Provider.of<AppStore>(context, listen: false);
      appStore.updateLoggedOwner(owner);

      delayAction().then(
        (value) => pushNamedAndRemoveUntil(context, AppRoutes.MAIN_PAGE),
      );
    }).catchError((err) {
      _blockUiStreamController.add(false);

      if ((err as String).isNotEmpty) {
        delayAction().then(
          (value) => genericErrorAlert(
            context: context,
            okActionText: 'OK',
            content: Text(err),
          ),
        );
      }
    });
  }

  @override
  void goToHomePageIfHasBeenAuthenticated(BuildContext context) async {
    if (await this._business.isAlreadyAuthenticated()) {
      pushNamedAndRemoveUntil(context, AppRoutes.MAIN_PAGE);
    }
  }

  @override
  void dispose() {
    this._emailCtrl.dispose();
    this._passwordCtrl.dispose();
    this._blockUiStreamController.close();
  }
}
