import 'package:flutter/material.dart';
import 'package:geolocator/geolocator.dart';
import 'package:onyva_mobile/models/repair_shop_search_result.dart';
import 'package:onyva_mobile/business/implementation/repair_shop_business_impl.dart';
import 'package:onyva_mobile/business/repair_shop_business.dart';
import 'package:onyva_mobile/models/repair_shop.dart';
import 'package:onyva_mobile/models/vehicle.dart';
import 'package:onyva_mobile/state_managers/app_store.dart';
import 'package:onyva_mobile/state_managers/repair_shop_search/repair_shop_search_store.dart';
import 'package:onyva_mobile/viewmodel/repair_shop_search_viewmodel.dart';
import 'package:provider/provider.dart';

class RepairShopSearchViewModelImpl implements RepairShopSearchViewModel {
  final RepairShopSearchStore _store = RepairShopSearchStore();
  final RepairShopBusiness _business = RepairShopBusinessImpl();

  @override
  List<RepairShop> get resultRepairShops => _store.resultRepairShops;

  @override
  bool get isBusy => _store.isBusy;

  @override
  RepairShop get selectedRepairShop => _store.selectedRepairShop;

  @override
  List<Map<String, dynamic>> get services => _store.services;

  @override
  bool get showFilters => _store.showFilters;

  @override
  void selectRepairShop(RepairShop repairShop) {
    _store.updateSelectedRepairShopState(repairShop);
  }

  @override
  void updateServices(
      BuildContext context, int index, Map<String, dynamic> newServices) {
    if (index >= 0 && index < services.length) {
      services[index] = newServices;
      _store.updateServices(services);
    }
    updateRepairShops(context);
  }

  @override
  void updateRepairShops(BuildContext context) async {
    _store.updateBusyState(true);

    AppStore appStore = Provider.of<AppStore>(context, listen: false);

    Position position;
    try {
      position =
          await getCurrentPosition(desiredAccuracy: LocationAccuracy.high);
    } on LocationServiceDisabledException {
      return cancel(context);
    }

    if (position == null) return;

    double latitude = position.latitude;
    double longitude = position.longitude;

    selectRepairShop(null);
    List<RepairShop> resultRs =
        await _business.getRepairShops(getSelectedServices());

    _store.updateResultRepairShopsState(resultRs);

    _store.updateBusyState(false);
  }

  @override
  void processArgs(Map<String, dynamic> args) {
    if (args != null) {
      bool showFiltersArgs = args['showFilters'];
      if (showFiltersArgs != null) _store.updateShowFilters(showFiltersArgs);
      List<String> servicesListArgs = args['services'];
      if (servicesListArgs != null) {
        services.addAll(servicesListArgs.map((s) => {
              'name': s,
              'selected': true,
            }));
        _store.updateServices(services);
      }
    }
  }

  @override
  void loadAllPossibleFilters(BuildContext context) async {
    AppStore appStore = Provider.of<AppStore>(context, listen: false);

    Vehicle vehicle = appStore.currentVehicle;

    List<String> newServices = await _business.getServicesByBrandModel(
        vehicle.fipeVehicle.brand, vehicle.fipeVehicle.model);

    services.addAll(newServices.map((s) => {
          'name': s,
          'selected': false,
        }));

    _store.updateServices(services);
  }

  @override
  void cancel(BuildContext context) {
    Navigator.pop(context, null);
  }

  List<String> getSelectedServices() {
    return services
        .where((sp) => sp['selected'] == true)
        .map((sp) => sp['name'].toString())
        .toList();
  }

  @override
  void confirm(BuildContext context) {
    List<String> services = getSelectedServices();
    RepairShopSearchResult result = new RepairShopSearchResult(
      result: selectedRepairShop,
      serviceFilters: services,
    );
    Navigator.pop(context, result);
  }
}
