import 'dart:async';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:onyva_mobile/app_routes.dart';
import 'package:onyva_mobile/business/implementation/repair_shop_business_impl.dart';
import 'package:onyva_mobile/business/implementation/scheduled_maintenance_business_impl.dart';
import 'package:onyva_mobile/business/repair_shop_business.dart';
import 'package:onyva_mobile/business/scheduled_maintenance_business.dart';
import 'package:onyva_mobile/core/functions/general_information_alert.dart';
import 'package:onyva_mobile/core/utils/functions.dart';
import 'package:onyva_mobile/core/widgets/color_sets.dart';
import 'package:onyva_mobile/core/widgets/date_time_picker.dart';
import 'package:onyva_mobile/models/owner.dart';
import 'package:onyva_mobile/models/repair_shop.dart';
import 'package:onyva_mobile/models/repair_shop_search_result.dart';
import 'package:onyva_mobile/models/requested_maintenance.dart';
import 'package:onyva_mobile/models/scheduled_maintenance.dart';
import 'package:onyva_mobile/models/vehicle.dart';
import 'package:onyva_mobile/state_managers/app_store.dart';
import 'package:onyva_mobile/state_managers/scheduled_maintenance/scheduled_maintenance_store.dart';
import 'package:onyva_mobile/viewmodel/scheduled_maintenance_viewmodel.dart';
import 'package:provider/provider.dart';

class ScheduledMaintenanceViewModelImpl
    implements ScheduledMaintenanceViewModel {
  final _store = ScheduledMaintenanceStore();
  final ScheduledMaintenanceBusiness _business =
      ScheduledMaintenanceBusinessImpl();

  final RepairShopBusiness _repairShopBusiness = RepairShopBusinessImpl();
  final _maintenanceTitleCtrl = new TextEditingController();

  final _optionalServiceDetailCtrl = new TextEditingController();

  DateTimePicker _repairShopCalendar;

  ScheduledMaintenanceViewModelImpl() {
    _store.setCurrentDateTime(firstDateTime);
  }

  @override
  bool get isBusy => _store.isBusy;

  @override
  DateTimePicker get repairShopCalendar => _repairShopCalendar;

  @override
  DateTime get currentDateTime => _store.currentDateTime;

  @override
  DateTime get firstDateTime => DateTime.now().add(new Duration(days: 1));

  @override
  DateTime get lastDateTime => DateTime.now().add(new Duration(days: 60));

  @override
  RepairShop get repairShop => _store.repairShop;

  @override
  ManutentionType get manutentionType => _store.manutentionType;

  @override
  bool get showOptionalDetailsField => manutentionType == ManutentionType.ALL;

  @override
  List<String> get servicesList => _store.servicesList;

  @override
  TextEditingController get optionalServiceDetailCtrl =>
      _optionalServiceDetailCtrl;

  @override
  TextEditingController get maintenanceTitleCtrl => _maintenanceTitleCtrl;

  @override
  void updateBusyState(bool newState) {
    _store.isBusy = newState;
  }

  @override
  void setDateTime(DateTime dateTime) {
    _store.setCurrentDateTime(dateTime);
  }

  @override
  void processArgs(Map<String, dynamic> args, BuildContext context) {
    if (args != null) {
      ManutentionType mType = args['manutentionType'];
      if (mType != null) {
        _store.setManutentionType(mType);
      }

      if (args['expired']) {
        @override
        void run() {
          scheduleMicrotask(() {
            generalInformationAlert(
              context: context,
              title: 'Agendamento de Manutenção Programada',
              body: Text(
                  'A oficina não respondeu à sua solicitação no prazo de 24 horas, por este motivo sua solicitação foi cancelada.'),
              okText: 'Ok',
            );
          });
        }

        run();
      }
    }
  }

  @override
  void setMaintenanceType(ManutentionType manutentionType) {
    _store.setManutentionType(manutentionType);
  }

  @override
  void loadManutentionOfCurrentVehicle(BuildContext context) async {
    AppStore appStore = Provider.of<AppStore>(context, listen: false);
    Vehicle vehicle = appStore.currentVehicle;

    List<ScheduledMaintenance> scheduledMaintenances =
        await _business.getScheduledMaintenancesServicesByBrandModel(
            vehicle.fipeVehicle.brand, vehicle.fipeVehicle.model);

    if (scheduledMaintenances.length == 0) return;

    ScheduledMaintenance sm = scheduledMaintenances[0];

    Set<String> services = new Set();

    services.addAll(sm.inspectionItems.split(';'));
    services.addAll(sm.replaceItems.split(';'));

    servicesList.addAll(services);

    _store.setServicesList(servicesList);
  }

  void setTime(TimeOfDay timeOfDay) {
    final currentDateTime = _store.currentDateTime;
    _store.setCurrentDateTime(
      new DateTime(
        currentDateTime.year,
        currentDateTime.month,
        currentDateTime.day,
        timeOfDay.hour,
        timeOfDay.minute,
      ),
    );
  }

  @override
  void chooseRepairShop(BuildContext context) async {
    var args = {
      'showFilters': manutentionType == ManutentionType.ALL,
      'services': servicesList,
    };
    RepairShopSearchResult searchResult = await pushNamedWithArguments(
        context, AppRoutes.REPAIR_SHOP_SEARCH, args) as RepairShopSearchResult;

    if (searchResult == null || searchResult.result == null) return;

    RepairShop selectedRepairShop = searchResult.result;
    List<String> newServicesList = searchResult.serviceFilters;

    _store.setServicesList(newServicesList);
    _store.setRepairShop(selectedRepairShop);
  }

  @override
  void schedule(BuildContext context) async {
    AppStore appStore = Provider.of<AppStore>(context, listen: false);
    Vehicle vehicle = appStore.currentVehicle;
    Owner owner = appStore.loggedOwner;

    if (owner == null || vehicle == null || repairShop == null) return;

    updateBusyState(true);

    ScheduledMaintenance fixedScheduledMaintenance;

    if (_store.manutentionType == ManutentionType.SCHEDULED) {
      fixedScheduledMaintenance = ScheduledMaintenance(
        id: '5fb72a196c22f81488a52b72',
        indicatedMileage: 10000,
        indicatedPeriod: 12,
        inspectionItems:
            "Motor;Óleo da transmissão;Pastilhas e discos de freio;Fluido de freio/embreagem;Freio de estacionamento;Amortecedores;Pneus;Rodas;Cintos de segurança;Coifas homocineticas;Sistema elétrico;Sistema de iluminação;Lavadores e limpadores;Palhetas dos limpadores;Sistema de Arrefecimento;Guarnições e protetores de pó;Sistema de A/C;Carroceria e assoalho",
        replaceItems:
            "Filtro de combustível;Óleo do motor;Filtro de óleo;Anel bujão cárter",
        title: "Revisão 10.000Km ou 1 Ano",
      );
      _maintenanceTitleCtrl.text = fixedScheduledMaintenance.title;
    }

    _business
        .schedule(
      owner.id,
      vehicle.id,
      repairShop.id,
      _store.currentDateTime,
      _maintenanceTitleCtrl.text,
      manutentionType,
      scheduledMaintenanceId:
          fixedScheduledMaintenance?.id ?? fixedScheduledMaintenance,
    )
        .then((value) {
      updateBusyState(false);
      setMaintenanceType(ManutentionType.ALL);

      generalInformationAlert(
        context: context,
        title: 'Agendamento de ${getNameOfManutentionType(manutentionType)}',
        body: Text('Agendamento realizado com sucesso!'),
        okText: 'Voltar à página inicial',
        callbackFunction: (result) {
          pushNamedAndRemoveUntil(
            context,
            AppRoutes.MAIN_PAGE,
          );
        },
      );
    }).catchError((e) {
      print(e);
      updateBusyState(false);
    });
  }

  @override
  bool enableOrDisableDayInRepairShopCalendar(DateTime day) {
    return _store.repairShopCalendar.every((repairShopCalendarEvent) =>
        repairShopCalendarEvent.start.day != day.day ||
        repairShopCalendarEvent.start.month != day.month ||
        repairShopCalendarEvent.start.year != day.year);
  }

  @override
  DateTime getFirstAvailableDayInCalendar() {
    DateTime currentDate = DateTime.now();
    DateTime validDate;

    final dateTimesUnavailableInRepairShop =
        _store.repairShopCalendar.map((calendarEvent) => calendarEvent.start);

    while (validDate == null) {
      if (dateTimesUnavailableInRepairShop.every(
        (dateTime) =>
            dateTime.day != currentDate.day ||
            dateTime.month != currentDate.month ||
            dateTime.year != currentDate.year,
      )) {
        validDate = currentDate;
      } else {
        currentDate = currentDate.add(Duration(days: 1));
      }
    }

    return validDate;
  }

  @override
  Future<void> loadRepairShopCalendar() async {
    updateBusyState(true);
    _repairShopBusiness.getRepairShopCalendarById(0).then((repairShopCalendar) {
      this._store.updateRepairShopCalendar(repairShopCalendar);
      _repairShopCalendar = DateTimePicker(
        height: 45,
        textColor: ColorSets.darkGray,
        backgroundColor: ColorSets.lightGray,
        currentDateTime: getFirstAvailableDayInCalendar(),
        firstDateTime: getFirstAvailableDayInCalendar(),
        lastDateTime: lastDateTime,
        onChange: setDateTime,
        enableDay: enableOrDisableDayInRepairShopCalendar,
      );
      updateBusyState(false);
    }).catchError((e) {
      updateBusyState(false);
      print(e);
    });
  }
}
